#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import inspect
import sys
import traceback

from . import command


class HelpAction(argparse.Action):
    """Provide a custom action so the -h and --help options
    to the main app will print a list of the commands.

    The commands are determined by checking the CommandManager
    instance, passed in as the "default" value for the action.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        app = self.default
        parser.print_help(app.stdout)
        app.stdout.write('\nCommands:\n')
        dists_by_module = command._get_distributions_by_modules()

        def dist_for_obj(obj):
            name = inspect.getmodule(obj).__name__.partition('.')[0]
            return dists_by_module.get(name)

        app_dist = dist_for_obj(app)
        command_manager = app.command_manager
        for name, ep in sorted(command_manager):
            try:
                factory = ep.load()
            except Exception:
                app.stdout.write('Could not load %r\n' % ep)
                if namespace.debug:
                    traceback.print_exc(file=app.stdout)
                continue
            try:
                kwargs = {}
                if 'cmd_name' in inspect.getargspec(factory.__init__).args:
                    kwargs['cmd_name'] = name
                cmd = factory(app, None, **kwargs)
                if cmd.deprecated:
                    continue
            except Exception as err:
                app.stdout.write('Could not instantiate %r: %s\n' % (ep, err))
                if namespace.debug:
                    traceback.print_exc(file=app.stdout)
                continue
            one_liner = cmd.get_description().split('\n')[0]
            dist_name = dist_for_obj(factory)
            if dist_name and dist_name != app_dist:
                dist_info = ' (' + dist_name + ')'
            else:
                dist_info = ''
            app.stdout.write('  %-13s  %s%s\n' % (name, one_liner, dist_info))
        sys.exit(0)


class HelpCommand(command.Command):
    """print detailed help for another command
    """

    def get_parser(self, prog_name):
        parser = super(HelpCommand, self).get_parser(prog_name)
        parser.add_argument('cmd',
                            nargs='*',
                            help='name of the command',
                            )
        return parser

    def take_action(self, parsed_args):
        if parsed_args.cmd:
            try:
                the_cmd = self.app.command_manager.find_command(
                    parsed_args.cmd,
                )
                cmd_factory, cmd_name, search_args = the_cmd
            except ValueError:
                # Did not find an exact match
                cmd = parsed_args.cmd[0]
                fuzzy_matches = [k[0] for k in self.app.command_manager
                                 if k[0].startswith(cmd)
                                 ]
                if not fuzzy_matches:
                    raise
                self.app.stdout.write('Command "%s" matches:\n' % cmd)
                for fm in sorted(fuzzy_matches):
                    self.app.stdout.write('  %s\n' % fm)
                return
            self.app_args.cmd = search_args
            kwargs = {}
            if 'cmd_name' in inspect.getargspec(cmd_factory.__init__).args:
                kwargs['cmd_name'] = cmd_name
            cmd = cmd_factory(self.app, self.app_args, **kwargs)
            full_name = (cmd_name
                         if self.app.interactive_mode
                         else ' '.join([self.app.NAME, cmd_name])
                         )
            cmd_parser = cmd.get_parser(full_name)
            cmd_parser.print_help(self.app.stdout)
        else:
            action = HelpAction(None, None, default=self.app)
            action(self.app.parser, self.app.options, None, None)
        return 0



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Output formatters values only
"""

import six

from . import base
from cliff import columns


class ValueFormatter(base.ListFormatter, base.SingleFormatter):

    def add_argument_group(self, parser):
        pass

    def emit_list(self, column_names, data, stdout, parsed_args):
        for row in data:
            stdout.write(
                ' '.join(
                    six.text_type(c.machine_readable()
                                  if isinstance(c, columns.FormattableColumn)
                                  else c)
                    for c in row) + u'\n')
        return

    def emit_one(self, column_names, data, stdout, parsed_args):
        for value in data:
            stdout.write('%s\n' % six.text_type(
                value.machine_readable()
                if isinstance(value, columns.FormattableColumn)
                else value)
            )
        return



# -*- encoding: utf-8 -*-
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import codecs
import locale
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import sys

import mock
import six

from cliff import app as application
from cliff import command as c_cmd
from cliff import commandmanager
from cliff.tests import base
from cliff.tests import utils as test_utils
from cliff import utils


def make_app(**kwargs):
    cmd_mgr = commandmanager.CommandManager('cliff.tests')

    # Register a command that succeeds
    command = mock.MagicMock(spec=c_cmd.Command)
    command_inst = mock.MagicMock(spec=c_cmd.Command)
    command_inst.run.return_value = 0
    command.return_value = command_inst
    cmd_mgr.add_command('mock', command)

    # Register a command that fails
    err_command = mock.Mock(name='err_command', spec=c_cmd.Command)
    err_command_inst = mock.Mock(spec=c_cmd.Command)
    err_command_inst.run = mock.Mock(
        side_effect=RuntimeError('test exception')
    )
    err_command.return_value = err_command_inst
    cmd_mgr.add_command('error', err_command)

    app = application.App('testing interactive mode',
                          '1',
                          cmd_mgr,
                          stderr=mock.Mock(),  # suppress warning messages
                          **kwargs
                          )
    return app, command


class TestInteractiveMode(base.TestBase):

    def test_no_args_triggers_interactive_mode(self):
        app, command = make_app()
        app.interact = mock.MagicMock(name='inspect')
        app.run([])
        app.interact.assert_called_once_with()

    def test_interactive_mode_cmdloop(self):
        app, command = make_app()
        app.interactive_app_factory = mock.MagicMock(
            name='interactive_app_factory'
        )
        self.assertIsNone(app.interpreter)
        app.run([])
        self.assertIsNotNone(app.interpreter)
        cmdloop = app.interactive_app_factory.return_value.cmdloop
        cmdloop.assert_called_once_with()


class TestInitAndCleanup(base.TestBase):

    def test_initialize_app(self):
        app, command = make_app()
        app.initialize_app = mock.MagicMock(name='initialize_app')
        app.run(['mock'])
        app.initialize_app.assert_called_once_with(['mock'])

    def test_prepare_to_run_command(self):
        app, command = make_app()
        app.prepare_to_run_command = mock.MagicMock(
            name='prepare_to_run_command',
        )
        app.run(['mock'])
        app.prepare_to_run_command.assert_called_once_with(command())

    def test_clean_up_success(self):
        app, command = make_app()
        app.clean_up = mock.MagicMock(name='clean_up')
        app.run(['mock'])
        app.clean_up.assert_called_once_with(command.return_value, 0, None)

    def test_clean_up_error(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(name='clean_up')
        app.run(['error'])

        app.clean_up.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 1, mock.ANY), call_args)
        args, kwargs = call_args
        self.assertIsInstance(args[2], RuntimeError)
        self.assertEqual(('test exception',), args[2].args)

    def test_clean_up_error_debug(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(name='clean_up')
        try:
            app.run(['--debug', 'error'])
        except RuntimeError as err:
            self.assertIs(err, app.clean_up.call_args_list[0][0][2])
        else:
            self.fail('Should have had an exception')

        self.assertTrue(app.clean_up.called)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 1, mock.ANY), call_args)
        args, kwargs = call_args
        self.assertIsInstance(args[2], RuntimeError)
        self.assertEqual(('test exception',), args[2].args)

    def test_error_handling_clean_up_raises_exception(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(
            name='clean_up',
            side_effect=RuntimeError('within clean_up'),
        )
        app.run(['error'])

        self.assertTrue(app.clean_up.called)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 1, mock.ANY), call_args)
        args, kwargs = call_args
        self.assertIsInstance(args[2], RuntimeError)
        self.assertEqual(('test exception',), args[2].args)

    def test_error_handling_clean_up_raises_exception_debug(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(
            name='clean_up',
            side_effect=RuntimeError('within clean_up'),
        )
        try:
            app.run(['--debug', 'error'])
        except RuntimeError as err:
            if not hasattr(err, '__context__'):
                # The exception passed to clean_up is not the exception
                # caused *by* clean_up.  This test is only valid in python
                # 2 because under v3 the original exception is re-raised
                # with the new one as a __context__ attribute.
                self.assertIsNot(err, app.clean_up.call_args_list[0][0][2])
        else:
            self.fail('Should have had an exception')

        self.assertTrue(app.clean_up.called)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 1, mock.ANY), call_args)
        args, kwargs = call_args
        self.assertIsInstance(args[2], RuntimeError)
        self.assertEqual(('test exception',), args[2].args)

    def test_normal_clean_up_raises_exception(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(
            name='clean_up',
            side_effect=RuntimeError('within clean_up'),
        )
        app.run(['mock'])

        self.assertTrue(app.clean_up.called)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 0, None), call_args)

    def test_normal_clean_up_raises_exception_debug(self):
        app, command = make_app()

        app.clean_up = mock.MagicMock(
            name='clean_up',
            side_effect=RuntimeError('within clean_up'),
        )
        app.run(['--debug', 'mock'])

        self.assertTrue(app.clean_up.called)
        call_args = app.clean_up.call_args_list[0]
        self.assertEqual(mock.call(mock.ANY, 0, None), call_args)


class TestOptionParser(base.TestBase):

    def test_conflicting_option_should_throw(self):
        class MyApp(application.App):
            def __init__(self):
                super(MyApp, self).__init__(
                    description='testing',
                    version='0.1',
                    command_manager=commandmanager.CommandManager('tests'),
                )

            def build_option_parser(self, description, version):
                parser = super(MyApp, self).build_option_parser(description,
                                                                version)
                parser.add_argument(
                    '-h', '--help',
                    default=self,  # tricky
                    help="Show help message and exit.",
                )

        self.assertRaises(
            argparse.ArgumentError,
            MyApp,
        )

    def test_conflicting_option_custom_arguments_should_not_throw(self):
        class MyApp(application.App):
            def __init__(self):
                super(MyApp, self).__init__(
                    description='testing',
                    version='0.1',
                    command_manager=commandmanager.CommandManager('tests'),
                )

            def build_option_parser(self, description, version):
                argparse_kwargs = {'conflict_handler': 'resolve'}
                parser = super(MyApp, self).build_option_parser(
                    description,
                    version,
                    argparse_kwargs=argparse_kwargs)
                parser.add_argument(
                    '-h', '--help',
                    default=self,  # tricky
                    help="Show help message and exit.",
                )

        MyApp()

    def test_option_parser_abbrev_issue(self):
        class MyCommand(c_cmd.Command):
            def get_parser(self, prog_name):
                parser = super(MyCommand, self).get_parser(prog_name)
                parser.add_argument("--end")
                return parser

            def take_action(self, parsed_args):
                assert(parsed_args.end == '123')

        class MyCommandManager(commandmanager.CommandManager):
            def load_commands(self, namespace):
                self.add_command("mycommand", MyCommand)

        class MyApp(application.App):
            def __init__(self):
                super(MyApp, self).__init__(
                    description='testing',
                    version='0.1',
                    command_manager=MyCommandManager(None),
                )

            def build_option_parser(self, description, version):
                parser = super(MyApp, self).build_option_parser(
                    description,
                    version,
                    argparse_kwargs={'allow_abbrev': False})
                parser.add_argument('--endpoint')
                return parser

        app = MyApp()
        # NOTE(jd) --debug is necessary so assert in take_action()
        # raises correctly here
        app.run(['--debug', 'mycommand', '--end', '123'])


class TestHelpHandling(base.TestBase):

    def _test_help(self, deferred_help):
        app, _ = make_app(deferred_help=deferred_help)
        with mock.patch.object(app, 'initialize_app') as init:
            with mock.patch('cliff.help.HelpAction.__call__',
                            side_effect=SystemExit(0)) as helper:
                self.assertRaises(
                    SystemExit,
                    app.run,
                    ['--help'],
                )
                self.assertTrue(helper.called)
            self.assertEqual(deferred_help, init.called)

    def test_help(self):
        self._test_help(False)

    def test_deferred_help(self):
        self._test_help(True)

    def test_subcommand_help(self):
        app, _ = make_app(deferred_help=False)

        # Help is called immediately
        with mock.patch('cliff.help.HelpAction.__call__') as helper:
            app.run(['show', 'files', '--help'])

        self.assertTrue(helper.called)

    def test_subcommand_deferred_help(self):
        app, _ = make_app(deferred_help=True)

        # Show that provide_help_if_requested() did not show help and exit
        with mock.patch.object(app, 'run_subcommand') as helper:
            app.run(['show', 'files', '--help'])

        helper.assert_called_once_with(['help', 'show', 'files'])


class TestCommandLookup(base.TestBase):

    def test_unknown_cmd(self):
        app, command = make_app()
        self.assertEqual(2, app.run(['hell']))

    def test_unknown_cmd_debug(self):
        app, command = make_app()
        try:
            self.assertEqual(2, app.run(['--debug', 'hell']))
        except ValueError as err:
            self.assertIn("['hell']", str(err))

    def test_list_matching_commands(self):
        stdout = StringIO()
        app = application.App('testing', '1',
                              test_utils.TestCommandManager(
                                  test_utils.TEST_NAMESPACE),
                              stdout=stdout)
        app.NAME = 'test'
        try:
            self.assertEqual(2, app.run(['t']))
        except SystemExit:
            pass
        output = stdout.getvalue()
        self.assertIn("test: 't' is not a test command. See 'test --help'.",
                      output)
        self.assertIn('Did you mean one of these?', output)
        self.assertIn('three word command\n  two words\n', output)

    def test_fuzzy_no_commands(self):
        cmd_mgr = commandmanager.CommandManager('cliff.fuzzy')
        app = application.App('test', '1.0', cmd_mgr)
        cmd_mgr.commands = {}
        matches = app.get_fuzzy_matches('foo')
        self.assertEqual([], matches)

    def test_fuzzy_common_prefix(self):
        # searched string is a prefix of all commands
        cmd_mgr = commandmanager.CommandManager('cliff.fuzzy')
        app = application.App('test', '1.0', cmd_mgr)
        cmd_mgr.commands = {}
        cmd_mgr.add_command('user list', test_utils.TestCommand)
        cmd_mgr.add_command('user show', test_utils.TestCommand)
        matches = app.get_fuzzy_matches('user')
        self.assertEqual(['user list', 'user show'], matches)

    def test_fuzzy_same_distance(self):
        # searched string has the same distance to all commands
        cmd_mgr = commandmanager.CommandManager('cliff.fuzzy')
        app = application.App('test', '1.0', cmd_mgr)
        cmd_mgr.add_command('user', test_utils.TestCommand)
        for cmd in cmd_mgr.commands.keys():
            self.assertEqual(
                8,
                utils.damerau_levenshtein('node', cmd, utils.COST),
            )
        matches = app.get_fuzzy_matches('node')
        self.assertEqual(['complete', 'help', 'user'], matches)

    def test_fuzzy_no_prefix(self):
        # search by distance, no common prefix with any command
        cmd_mgr = commandmanager.CommandManager('cliff.fuzzy')
        app = application.App('test', '1.0', cmd_mgr)
        cmd_mgr.add_command('user', test_utils.TestCommand)
        matches = app.get_fuzzy_matches('uesr')
        self.assertEqual(['user'], matches)


class TestVerboseMode(base.TestBase):

    def test_verbose(self):
        app, command = make_app()
        app.clean_up = mock.MagicMock(name='clean_up')
        app.run(['--verbose', 'mock'])
        app.clean_up.assert_called_once_with(command.return_value, 0, None)
        app.clean_up.reset_mock()
        app.run(['--quiet', 'mock'])
        app.clean_up.assert_called_once_with(command.return_value, 0, None)
        self.assertRaises(
            SystemExit,
            app.run,
            ['--verbose', '--quiet', 'mock'],
        )


class TestIO(base.TestBase):

    def test_io_streams(self):
        cmd_mgr = commandmanager.CommandManager('cliff.tests')
        io = mock.Mock()

        if six.PY2:
            stdin_save = sys.stdin
            stdout_save = sys.stdout
            stderr_save = sys.stderr
            encoding = locale.getpreferredencoding() or 'utf-8'

            app = application.App('no io streams', 1, cmd_mgr)
            self.assertIsInstance(app.stdin, codecs.StreamReader)
            self.assertIsInstance(app.stdout, codecs.StreamWriter)
            self.assertIsInstance(app.stderr, codecs.StreamWriter)

            app = application.App('with stdin io stream', 1, cmd_mgr, stdin=io)
            self.assertIs(io, app.stdin)
            self.assertIsInstance(app.stdout, codecs.StreamWriter)
            self.assertIsInstance(app.stderr, codecs.StreamWriter)

            app = application.App('with stdout io stream', 1, cmd_mgr,
                                  stdout=io)
            self.assertIsInstance(app.stdin, codecs.StreamReader)
            self.assertIs(io, app.stdout)
            self.assertIsInstance(app.stderr, codecs.StreamWriter)

            app = application.App('with stderr io stream', 1, cmd_mgr,
                                  stderr=io)
            self.assertIsInstance(app.stdin, codecs.StreamReader)
            self.assertIsInstance(app.stdout, codecs.StreamWriter)
            self.assertIs(io, app.stderr)

            try:
                sys.stdin = codecs.getreader(encoding)(sys.stdin)
                app = application.App(
                    'with wrapped sys.stdin io stream', 1, cmd_mgr)
                self.assertIs(sys.stdin, app.stdin)
                self.assertIsInstance(app.stdout, codecs.StreamWriter)
                self.assertIsInstance(app.stderr, codecs.StreamWriter)
            finally:
                sys.stdin = stdin_save

            try:
                sys.stdout = codecs.getwriter(encoding)(sys.stdout)
                app = application.App('with wrapped stdout io stream', 1,
                                      cmd_mgr)
                self.assertIsInstance(app.stdin, codecs.StreamReader)
                self.assertIs(sys.stdout, app.stdout)
                self.assertIsInstance(app.stderr, codecs.StreamWriter)
            finally:
                sys.stdout = stdout_save

            try:
                sys.stderr = codecs.getwriter(encoding)(sys.stderr)
                app = application.App('with wrapped stderr io stream', 1,
                                      cmd_mgr)
                self.assertIsInstance(app.stdin, codecs.StreamReader)
                self.assertIsInstance(app.stdout, codecs.StreamWriter)
                self.assertIs(sys.stderr, app.stderr)
            finally:
                sys.stderr = stderr_save

        else:
            app = application.App('no io streams', 1, cmd_mgr)
            self.assertIs(sys.stdin, app.stdin)
            self.assertIs(sys.stdout, app.stdout)
            self.assertIs(sys.stderr, app.stderr)

            app = application.App('with stdin io stream', 1, cmd_mgr, stdin=io)
            self.assertIs(io, app.stdin)
            self.assertIs(sys.stdout, app.stdout)
            self.assertIs(sys.stderr, app.stderr)

            app = application.App('with stdout io stream', 1, cmd_mgr,
                                  stdout=io)
            self.assertIs(sys.stdin, app.stdin)
            self.assertIs(io, app.stdout)
            self.assertIs(sys.stderr, app.stderr)

            app = application.App('with stderr io stream', 1, cmd_mgr,
                                  stderr=io)
            self.assertIs(sys.stdin, app.stdin)
            self.assertIs(sys.stdout, app.stdout)
            self.assertIs(io, app.stderr)

    def test_writer_encoding(self):
        # The word "test" with the e replaced by
        # Unicode latin small letter e with acute,
        # U+00E9, utf-8 encoded as 0xC3 0xA9
        text = u't\u00E9st'
        text_utf8 = text.encode('utf-8')

        if six.PY2:
            # In PY2 StreamWriter can't accept non-ASCII encoded characters
            # because it must first promote the encoded byte stream to
            # unicode in order to encode it in the desired encoding.
            # Because the encoding of the byte stream is not known at this
            # point the default-encoding of ASCII is utilized, but you can't
            # decode a non-ASCII charcater to ASCII.
            io = six.StringIO()
            writer = codecs.getwriter('utf-8')(io)
            self.assertRaises(UnicodeDecodeError,
                              writer.write,
                              text_utf8)

            # In PY2 with our override of codecs.getwriter we do not
            # attempt to encode bytes in a str object (only unicode
            # objects) therefore the final output string should be the
            # utf-8 encoded byte sequence
            io = six.StringIO()
            writer = utils.getwriter('utf-8')(io)
            writer.write(text)
            output = io.getvalue()
            self.assertEqual(text_utf8, output)

            io = six.StringIO()
            writer = utils.getwriter('utf-8')(io)
            writer.write(text_utf8)
            output = io.getvalue()
            self.assertEqual(text_utf8, output)
        else:
            # In PY3 you can't write encoded bytes to a text writer
            # instead text functions require text.
            io = six.StringIO()
            writer = utils.getwriter('utf-8')(io)
            self.assertRaises(TypeError,
                              writer.write,
                              text)

            io = six.StringIO()
            writer = utils.getwriter('utf-8')(io)
            self.assertRaises(TypeError,
                              writer.write,
                              text_utf8)



# -*- encoding: utf-8 -*-

import logging

from cliff.lister import Lister


class Encoding(Lister):
    """Show some unicode text
    """

    log = logging.getLogger(__name__)

    def take_action(self, parsed_args):
        messages = [
            u'pi: π',
            u'GB18030:鼀丅㐀ٸཌྷᠧꌢ€',
        ]
        return (
            ('UTF-8', 'Unicode'),
            [(repr(t.encode('utf-8')), t)
             for t in messages],
        )



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import mock
import testscenarios

from cliff import command
from cliff import commandmanager
from cliff.tests import base
from cliff.tests import utils


load_tests = testscenarios.load_tests_apply_scenarios


class TestLookupAndFind(base.TestBase):

    scenarios = [
        ('one-word', {'argv': ['one']}),
        ('two-words', {'argv': ['two', 'words']}),
        ('three-words', {'argv': ['three', 'word', 'command']}),
    ]

    def test(self):
        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        cmd, name, remaining = mgr.find_command(self.argv)
        self.assertTrue(cmd)
        self.assertEqual(' '.join(self.argv), name)
        self.assertFalse(remaining)


class TestLookupWithRemainder(base.TestBase):

    scenarios = [
        ('one', {'argv': ['one', '--opt']}),
        ('two', {'argv': ['two', 'words', '--opt']}),
        ('three', {'argv': ['three', 'word', 'command', '--opt']}),
    ]

    def test(self):
        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        cmd, name, remaining = mgr.find_command(self.argv)
        self.assertTrue(cmd)
        self.assertEqual(['--opt'], remaining)


class TestFindInvalidCommand(base.TestBase):

    scenarios = [
        ('no-such-command', {'argv': ['a', '-b']}),
        ('no-command-given', {'argv': ['-b']}),
    ]

    def test(self):
        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        try:
            mgr.find_command(self.argv)
        except ValueError as err:
            # make sure err include 'a' when ['a', '-b']
            self.assertIn(self.argv[0], str(err))
            self.assertIn('-b', str(err))
        else:
            self.fail('expected a failure')


class TestFindUnknownCommand(base.TestBase):

    def test(self):
        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        try:
            mgr.find_command(['a', 'b'])
        except ValueError as err:
            self.assertIn("['a', 'b']", str(err))
        else:
            self.fail('expected a failure')


class TestDynamicCommands(base.TestBase):

    def test_add(self):
        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        mock_cmd = mock.Mock()
        mgr.add_command('mock', mock_cmd)
        found_cmd, name, args = mgr.find_command(['mock'])
        self.assertIs(mock_cmd, found_cmd)

    def test_intersected_commands(self):
        def foo(arg):
            pass

        def foo_bar():
            pass

        mgr = utils.TestCommandManager(utils.TEST_NAMESPACE)
        mgr.add_command('foo', foo)
        mgr.add_command('foo bar', foo_bar)

        self.assertIs(foo_bar, mgr.find_command(['foo', 'bar'])[0])
        self.assertIs(
            foo,
            mgr.find_command(['foo', 'arg0'])[0],
        )


class TestLoad(base.TestBase):

    def test_load_commands(self):
        testcmd = mock.Mock(name='testcmd')
        testcmd.name.replace.return_value = 'test'
        mock_pkg_resources = mock.Mock(return_value=[testcmd])
        with mock.patch('pkg_resources.iter_entry_points',
                        mock_pkg_resources) as iter_entry_points:
            mgr = commandmanager.CommandManager('test')
            iter_entry_points.assert_called_once_with('test')
            names = [n for n, v in mgr]
            self.assertEqual(['test'], names)

    def test_load_commands_keep_underscores(self):
        testcmd = mock.Mock()
        testcmd.name = 'test_cmd'
        mock_pkg_resources = mock.Mock(return_value=[testcmd])
        with mock.patch('pkg_resources.iter_entry_points',
                        mock_pkg_resources) as iter_entry_points:
            mgr = commandmanager.CommandManager(
                'test',
                convert_underscores=False,
            )
            iter_entry_points.assert_called_once_with('test')
            names = [n for n, v in mgr]
            self.assertEqual(['test_cmd'], names)

    def test_load_commands_replace_underscores(self):
        testcmd = mock.Mock()
        testcmd.name = 'test_cmd'
        mock_pkg_resources = mock.Mock(return_value=[testcmd])
        with mock.patch('pkg_resources.iter_entry_points',
                        mock_pkg_resources) as iter_entry_points:
            mgr = commandmanager.CommandManager(
                'test',
                convert_underscores=True,
            )
            iter_entry_points.assert_called_once_with('test')
            names = [n for n, v in mgr]
            self.assertEqual(['test cmd'], names)


class FauxCommand(command.Command):

    def take_action(self, parsed_args):
        return 0


class FauxCommand2(FauxCommand):
    pass


class TestLegacyCommand(base.TestBase):

    def test_find_legacy(self):
        mgr = utils.TestCommandManager(None)
        mgr.add_command('new name', FauxCommand)
        mgr.add_legacy_command('old name', 'new name')
        cmd, name, remaining = mgr.find_command(['old', 'name'])
        self.assertIs(cmd, FauxCommand)
        self.assertEqual(name, 'old name')

    def test_legacy_overrides_new(self):
        mgr = utils.TestCommandManager(None)
        mgr.add_command('cmd1', FauxCommand)
        mgr.add_command('cmd2', FauxCommand2)
        mgr.add_legacy_command('cmd2', 'cmd1')
        cmd, name, remaining = mgr.find_command(['cmd2'])
        self.assertIs(cmd, FauxCommand)
        self.assertEqual(name, 'cmd2')

    def test_no_legacy(self):
        mgr = utils.TestCommandManager(None)
        mgr.add_command('cmd1', FauxCommand)
        self.assertRaises(
            ValueError,
            mgr.find_command,
            ['cmd2'],
        )

    def test_no_command(self):
        mgr = utils.TestCommandManager(None)
        mgr.add_legacy_command('cmd2', 'cmd1')
        self.assertRaises(
            ValueError,
            mgr.find_command,
            ['cmd2'],
        )



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import abc
import inspect

import pkg_resources
import six
from stevedore import extension

from cliff import _argparse

_dists_by_mods = None


def _get_distributions_by_modules():
    """Return dict mapping module name to distribution names.

    The python package name (the name used for importing) and the
    distribution name (the name used with pip and PyPI) do not
    always match. We want to report which distribution caused the
    command to be installed, so we need to look up the values.

    """
    global _dists_by_mods
    if _dists_by_mods is None:
        results = {}
        for dist in pkg_resources.working_set:
            try:
                mod_names = dist.get_metadata('top_level.txt').strip()
            except Exception:
                # Could not retrieve metadata. Either the file is not
                # present or we cannot read it. Ignore the
                # distribution.
                pass
            else:
                # Distributions may include multiple top-level
                # packages (see setuptools for an example).
                for mod_name in mod_names.splitlines():
                    results[mod_name] = dist.project_name
        _dists_by_mods = results
    return _dists_by_mods


def _get_distribution_for_module(module):
    "Return the distribution containing the module."
    dist_name = None
    if module:
        pkg_name = module.__name__.partition('.')[0]
        dist_name = _get_distributions_by_modules().get(pkg_name)
    return dist_name


@six.add_metaclass(abc.ABCMeta)
class Command(object):
    """Base class for command plugins.

    When the command is instantiated, it loads extensions from a
    namespace based on the parent application namespace and the
    command name::

        app.namespace + '.' + cmd_name.replace(' ', '_')

    :param app: Application instance invoking the command.
    :paramtype app: cliff.app.App

    """

    deprecated = False

    _description = ''
    _epilog = None

    def __init__(self, app, app_args, cmd_name=None):
        self.app = app
        self.app_args = app_args
        self.cmd_name = cmd_name
        self._load_hooks()

    def _load_hooks(self):
        # Look for command extensions
        if self.app and self.cmd_name:
            namespace = '{}.{}'.format(
                self.app.command_manager.namespace,
                self.cmd_name.replace(' ', '_')
            )
            self._hooks = extension.ExtensionManager(
                namespace=namespace,
                invoke_on_load=True,
                invoke_kwds={
                    'command': self,
                },
            )
        else:
            # Setting _hooks to an empty list allows iteration without
            # checking if there are hooks every time.
            self._hooks = []
        return

    def get_description(self):
        """Return the command description.

        The default is to use the first line of the class' docstring
        as the description. Set the ``_description`` class attribute
        to a one-line description of a command to use a different
        value. This is useful for enabling translations, for example,
        with ``_description`` set to a string wrapped with a gettext
        translation marker.

        """
        # NOTE(dhellmann): We need the trailing "or ''" because under
        # Python 2.7 the default for the docstring is None instead of
        # an empty string, and we always want this method to return a
        # string.
        desc = self._description or inspect.getdoc(self.__class__) or ''
        # The base class command description isn't useful for any
        # real commands, so ignore that value.
        if desc == inspect.getdoc(Command):
            desc = ''
        return desc

    def get_epilog(self):
        """Return the command epilog."""
        # replace a None in self._epilog with an empty string
        parts = [self._epilog or '']
        hook_epilogs = filter(
            None,
            (h.obj.get_epilog() for h in self._hooks),
        )
        parts.extend(hook_epilogs)
        app_dist_name = _get_distribution_for_module(
            inspect.getmodule(self.app)
        )
        dist_name = _get_distribution_for_module(inspect.getmodule(self))
        if dist_name and dist_name != app_dist_name:
            parts.append(
                'This command is provided by the %s plugin.' %
                (dist_name,)
            )
        return '\n\n'.join(parts)

    def get_parser(self, prog_name):
        """Return an :class:`argparse.ArgumentParser`.
        """
        parser = _argparse.ArgumentParser(
            description=self.get_description(),
            epilog=self.get_epilog(),
            prog=prog_name,
            formatter_class=_SmartHelpFormatter,
        )
        for hook in self._hooks:
            hook.obj.get_parser(parser)
        return parser

    @abc.abstractmethod
    def take_action(self, parsed_args):
        """Override to do something useful.

        The returned value will be returned by the program.
        """

    def run(self, parsed_args):
        """Invoked by the application when the command is run.

        Developers implementing commands should override
        :meth:`take_action`.

        Developers creating new command base classes (such as
        :class:`Lister` and :class:`ShowOne`) should override this
        method to wrap :meth:`take_action`.

        Return the value returned by :meth:`take_action` or 0.
        """
        parsed_args = self._run_before_hooks(parsed_args)
        return_code = self.take_action(parsed_args) or 0
        return_code = self._run_after_hooks(parsed_args, return_code)
        return return_code

    def _run_before_hooks(self, parsed_args):
        """Calls before() method of the hooks.

        This method is intended to be called from the run() method before
        take_action() is called.

        This method should only be overriden by developers creating new
        command base classes and only if it is necessary to have different
        hook processing behavior.
        """
        for hook in self._hooks:
            ret = hook.obj.before(parsed_args)
            # If the return is None do not change parsed_args, otherwise
            # set up to pass it to the next hook
            if ret is not None:
                parsed_args = ret
        return parsed_args

    def _run_after_hooks(self, parsed_args, return_code):
        """Calls after() method of the hooks.

        This method is intended to be called from the run() method after
        take_action() is called.

        This method should only be overriden by developers creating new
        command base classes and only if it is necessary to have different
        hook processing behavior.
        """
        for hook in self._hooks:
            ret = hook.obj.after(parsed_args, return_code)
            # If the return is None do not change return_code, otherwise
            # set up to pass it to the next hook
            if ret is not None:
                return_code = ret
        return return_code


class _SmartHelpFormatter(_argparse.HelpFormatter):
    """Smart help formatter to output raw help message if help contain \n.

    Some command help messages maybe have multiple line content, the built-in
    argparse.HelpFormatter wrap and split the content according to width, and
    ignore \n in the raw help message, it merge multiple line content in one
    line to output, that looks messy. SmartHelpFormatter keep the raw help
    message format if it contain \n, and wrap long line like HelpFormatter
    behavior.
    """

    def _split_lines(self, text, width):
        lines = text.splitlines() if '\n' in text else [text]
        wrap_lines = []
        for each_line in lines:
            wrap_lines.extend(
                super(_SmartHelpFormatter, self)._split_lines(each_line, width)
            )
        return wrap_lines



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Special argparse module that allows to bypass abbrev mode."""

from __future__ import absolute_import
from argparse import *  # noqa
import sys


if sys.version_info < (3, 5):
    class ArgumentParser(ArgumentParser):  # noqa
        def __init__(self, *args, **kwargs):
            self.allow_abbrev = kwargs.pop("allow_abbrev", True)
            super(ArgumentParser, self).__init__(*args, **kwargs)

        def _get_option_tuples(self, option_string):
            if self.allow_abbrev:
                return super(ArgumentParser, self)._get_option_tuples(
                    option_string)
            return ()



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Bash completion tests
"""

import mock

from cliff import app as application
from cliff import commandmanager
from cliff import complete
from cliff.tests import base


class TestCompletion(base.TestBase):

    def test_dictionary(self):
        sot = complete.CompleteDictionary()
        sot.add_command("image delete".split(),
                        [mock.Mock(option_strings=["1"])])
        sot.add_command("image list".split(),
                        [mock.Mock(option_strings=["2"])])
        sot.add_command("image create".split(),
                        [mock.Mock(option_strings=["3"])])
        sot.add_command("volume type create".split(),
                        [mock.Mock(option_strings=["4"])])
        sot.add_command("volume type delete".split(),
                        [mock.Mock(option_strings=["5"])])
        self.assertEqual("image volume", sot.get_commands())
        result = sot.get_data()
        self.assertEqual("image", result[0][0])
        self.assertEqual("create delete list", result[0][1])
        self.assertEqual("image_create", result[1][0])
        self.assertEqual("3", result[1][1])
        self.assertEqual("image_delete", result[2][0])
        self.assertEqual("1", result[2][1])
        self.assertEqual("image_list", result[3][0])
        self.assertEqual("2", result[3][1])

    def test_complete_dictionary_subcmd(self):
        sot = complete.CompleteDictionary()
        sot.add_command("image delete".split(),
                        [mock.Mock(option_strings=["1"])])
        sot.add_command("image list".split(),
                        [mock.Mock(option_strings=["2"])])
        sot.add_command("image list better".split(),
                        [mock.Mock(option_strings=["3"])])
        self.assertEqual("image", sot.get_commands())
        result = sot.get_data()
        self.assertEqual("image", result[0][0])
        self.assertEqual("delete list list_better", result[0][1])
        self.assertEqual("image_delete", result[1][0])
        self.assertEqual("1", result[1][1])
        self.assertEqual("image_list", result[2][0])
        self.assertEqual("2 better", result[2][1])
        self.assertEqual("image_list_better", result[3][0])
        self.assertEqual("3", result[3][1])


class FakeStdout:
    def __init__(self):
        self.content = []

    def write(self, text):
        self.content.append(text)

    def make_string(self):
        result = ''
        for line in self.content:
            result = result + line
        return result


class TestCompletionAlternatives(base.TestBase):

    def given_cmdo_data(self):
        cmdo = "image server"
        data = [("image", "create"),
                ("image_create", "--eolus"),
                ("server", "meta ssh"),
                ("server_meta_delete", "--wilson"),
                ("server_ssh", "--sunlight")]
        return cmdo, data

    def then_data(self, content):
        self.assertIn("  cmds='image server'\n", content)
        self.assertIn("  cmds_image='create'\n", content)
        self.assertIn("  cmds_image_create='--eolus'\n", content)
        self.assertIn("  cmds_server='meta ssh'\n", content)
        self.assertIn("  cmds_server_meta_delete='--wilson'\n", content)
        self.assertIn("  cmds_server_ssh='--sunlight'\n", content)

    def test_complete_no_code(self):
        output = FakeStdout()
        sot = complete.CompleteNoCode("doesNotMatter", output)
        sot.write(*self.given_cmdo_data())
        self.then_data(output.content)

    def test_complete_bash(self):
        output = FakeStdout()
        sot = complete.CompleteBash("openstack", output)
        sot.write(*self.given_cmdo_data())
        self.then_data(output.content)
        self.assertIn("_openstack()\n", output.content[0])
        self.assertIn("complete -F _openstack openstack\n", output.content[-1])

    def test_complete_command_parser(self):
        sot = complete.CompleteCommand(mock.Mock(), mock.Mock())
        parser = sot.get_parser('nothing')
        self.assertEqual("nothing", parser.prog)
        self.assertEqual("print bash completion command\n    ",
                         parser.description)


class TestCompletionAction(base.TestBase):

    def given_complete_command(self):
        cmd_mgr = commandmanager.CommandManager('cliff.tests')
        app = application.App('testing', '1', cmd_mgr, stdout=FakeStdout())
        sot = complete.CompleteCommand(app, mock.Mock())
        cmd_mgr.add_command('complete', complete.CompleteCommand)
        return sot, app, cmd_mgr

    def then_actions_equal(self, actions):
        optstr = ' '.join(opt for action in actions
                          for opt in action.option_strings)
        self.assertEqual('-h --help --name --shell', optstr)

    def test_complete_command_get_actions(self):
        sot, app, cmd_mgr = self.given_complete_command()
        app.interactive_mode = False
        actions = sot.get_actions(["complete"])
        self.then_actions_equal(actions)

    def test_complete_command_get_actions_interactive(self):
        sot, app, cmd_mgr = self.given_complete_command()
        app.interactive_mode = True
        actions = sot.get_actions(["complete"])
        self.then_actions_equal(actions)

    def test_complete_command_take_action(self):
        sot, app, cmd_mgr = self.given_complete_command()
        parsed_args = mock.Mock()
        parsed_args.name = "test_take"
        parsed_args.shell = "bash"
        content = app.stdout.content
        self.assertEqual(0, sot.take_action(parsed_args))
        self.assertIn("_test_take()\n", content[0])
        self.assertIn("complete -F _test_take test_take\n", content[-1])
        self.assertIn("  cmds='complete help'\n", content)
        self.assertIn("  cmds_complete='-h --help --name --shell'\n", content)
        self.assertIn("  cmds_help='-h --help'\n", content)

    def test_complete_command_remove_dashes(self):
        sot, app, cmd_mgr = self.given_complete_command()
        parsed_args = mock.Mock()
        parsed_args.name = "test-take"
        parsed_args.shell = "bash"
        content = app.stdout.content
        self.assertEqual(0, sot.take_action(parsed_args))
        self.assertIn("_test_take()\n", content[0])
        self.assertIn("complete -F _test_take test-take\n", content[-1])



import sys

from cliff.app import App
from cliff.commandmanager import CommandManager


class DemoApp(App):

    def __init__(self):
        super(DemoApp, self).__init__(
            description='cliff demo app',
            version='0.1',
            command_manager=CommandManager('cliff.demo'),
            deferred_help=True,
            )

    def initialize_app(self, argv):
        self.LOG.debug('initialize_app')

    def prepare_to_run_command(self, cmd):
        self.LOG.debug('prepare_to_run_command %s', cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.LOG.debug('clean_up %s', cmd.__class__.__name__)
        if err:
            self.LOG.debug('got an error: %s', err)


def main(argv=sys.argv[1:]):
    myapp = DemoApp()
    return myapp.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Output formatters using shell syntax.
"""

from . import base
from cliff import columns

import argparse
import six


class ShellFormatter(base.SingleFormatter):

    def add_argument_group(self, parser):
        group = parser.add_argument_group(
            title='shell formatter',
            description='a format a UNIX shell can parse (variable="value")',
        )
        group.add_argument(
            '--variable',
            action='append',
            default=[],
            dest='variables',
            metavar='VARIABLE',
            help=argparse.SUPPRESS,
        )
        group.add_argument(
            '--prefix',
            action='store',
            default='',
            dest='prefix',
            help='add a prefix to all variable names',
        )

    def emit_one(self, column_names, data, stdout, parsed_args):
        variable_names = [c.lower().replace(' ', '_')
                          for c in column_names
                          ]
        desired_columns = parsed_args.variables
        for name, value in zip(variable_names, data):
            if name in desired_columns or not desired_columns:
                value = (six.text_type(value.machine_readable())
                         if isinstance(value, columns.FormattableColumn)
                         else value)
                if isinstance(value, six.string_types):
                    value = value.replace('"', '\\"')
                if isinstance(name, six.string_types):
                    # Colons and dashes may appear as a resource property but
                    # are invalid to use in a shell, replace them with an
                    # underscore.
                    name = name.replace(':', '_')
                    name = name.replace('-', '_')
                stdout.write('%s%s="%s"\n' % (parsed_args.prefix, name, value))
        return



# -*- encoding: utf-8 -*-
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import cmd2

from cliff.interactive import InteractiveApp
from cliff.tests import base


class FakeApp(object):
    NAME = 'Fake'


class TestInteractive(base.TestBase):

    def make_interactive_app(self, *command_names):
        fake_command_manager = [(x, None) for x in command_names]
        return InteractiveApp(FakeApp, fake_command_manager,
                              stdin=None, stdout=None)

    def _test_completenames(self, expecteds, prefix):
        app = self.make_interactive_app('hips', 'hippo', 'nonmatching')
        self.assertEqual(
            set(app.completenames(prefix, '', 0, 1)), set(expecteds))

    def test_cmd2_completenames(self):
        # cmd2.Cmd define do_help method
        self._test_completenames(['help'], 'he')

    def test_cliff_completenames(self):
        self._test_completenames(['hips', 'hippo'], 'hip')

    def test_no_completenames(self):
        self._test_completenames([], 'taz')

    def test_both_completenames(self):
        # cmd2.Cmd define do_history method
        # NOTE(dtroyer): Before release 0.7.0 do_hi was also defined so we need
        #                to account for that in the list of possible responses.
        #                Remove this check after cmd2 0.7.0 is the minimum
        #                requirement.
        if hasattr(cmd2.Cmd, "do_hi"):
            self._test_completenames(['hi', 'history', 'hips', 'hippo'], 'hi')
        else:
            self._test_completenames(['history', 'hips', 'hippo'], 'hi')

    def _test_completedefault(self, expecteds, line, begidx):
        command_names = set(['show file', 'show folder', 'show  long',
                             'list all'])
        app = self.make_interactive_app(*command_names)
        observeds = app.completedefault(None, line, begidx, None)
        self.assertEqual(set(expecteds), set(observeds))
        self.assertTrue(
            set([line[:begidx] + x for x in observeds]) <= command_names
        )

    def test_empty_text_completedefault(self):
        # line = 'show ' + begidx = 5 implies text = ''
        self._test_completedefault(['file', 'folder', ' long'], 'show ', 5)

    def test_nonempty_text_completedefault2(self):
        # line = 'show f' + begidx = 6 implies text = 'f'
        self._test_completedefault(['file', 'folder'], 'show f', 5)

    def test_long_completedefault(self):
        self._test_completedefault(['long'], 'show  ', 6)

    def test_no_completedefault(self):
        self._test_completedefault([], 'taz ', 4)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import abc

import six


@six.add_metaclass(abc.ABCMeta)
class CommandHook(object):
    """Base class for command hooks.

    :param app: Command instance being invoked
    :paramtype app: cliff.command.Command

    """

    def __init__(self, command):
        self.cmd = command

    @abc.abstractmethod
    def get_parser(self, parser):
        """Return an :class:`argparse.ArgumentParser`.

        :param parser: An existing ArgumentParser instance to be modified.
        :paramtype parser: ArgumentParser
        :returns: ArgumentParser
        """
        return parser

    @abc.abstractmethod
    def get_epilog(self):
        "Return text to add to the command help epilog."
        return ''

    @abc.abstractmethod
    def before(self, parsed_args):
        """Called before the command's take_action() method.

        :param parsed_args: The arguments to the command.
        :paramtype parsed_args: argparse.Namespace
        :returns: argparse.Namespace
        """
        return parsed_args

    @abc.abstractmethod
    def after(self, parsed_args, return_code):
        """Called after the command's take_action() method.

        :param parsed_args: The arguments to the command.
        :paramtype parsed_args: argparse.Namespace
        :param return_code: The value returned from take_action().
        :paramtype return_code: int
        :returns: int
        """
        return return_code
